/* OpenCMISS-Cmgui Application
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*******************************************************************************
FILE : process_list_or_write_command.hpp

LAST MODIFIED : 15 August 2007

DESCRIPTION :
Class built for gfx write all and gfx list all_commands. This class
will process either list commands or write to comfile commands
==============================================================================*/
#if !defined (PROCESS_LIST_OR_WRITE_COMMAND_H)
#define PROCESS_LIST_OR_WRITE_COMMAND_H

#include <stdarg.h>

#define MESSAGE_STRING_SIZE 1000
static char message_string[MESSAGE_STRING_SIZE];

class Process_list_or_write_command_class
{

public:
	 Process_list_or_write_command_class()
	 {
	 };

	 virtual int write_enabled() = 0;

	 virtual int process_command(enum Message_type message_type,const char *format,...) = 0;

	 virtual ~Process_list_or_write_command_class() { }
};

class Process_list_command_class : public Process_list_or_write_command_class
{
public:
	 Process_list_command_class()
	 {
	 };

	 int process_command(enum Message_type message_type,const char *format,...)
	 {
			int return_code;
			va_list ap;
			va_start(ap, format);
			vsprintf(message_string,format,ap);
			return_code = display_message(message_type, "%s", message_string);
			va_end(ap);
			return (return_code);
	 }

	 int write_enabled()
	{
		 int return_code = 0;
		 return (return_code);
	}

};

class Process_write_command_class : public Process_list_or_write_command_class
{

public:
	 Process_write_command_class()
	 {
	 };

	 int process_command(enum Message_type message_type,const char *format,...)
	 {
			int return_code;
			va_list ap;
			va_start(ap, format);
			vsprintf(message_string,format,ap);
			return_code = write_message_to_file(message_type, message_string,"");
			va_end(ap);
			return (return_code);
	 }

	 int write_enabled()
	{
		 int return_code = 1;
		 return (return_code);
	}
};

#endif /* define PROCESS_LIST_OR_WRITE_COMMAND_H */
